@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpctl.c,v 1.262 2020/05/02 14:33:33 claudio Exp $ */
+/*	$OpenBSD: bgpctl.c,v 1.268 2021/05/27 08:29:06 claudio Exp $ */
 
 /*
  * Copyright (c) 2003 Henning Brauer <henning@openbsd.org>
@@ -120,7 +120,7 @@ main(int argc, char *argv[])
 	memcpy(&neighbor.addr, &res->peeraddr, sizeof(neighbor.addr));
 	strlcpy(neighbor.descr, res->peerdesc, sizeof(neighbor.descr));
 	neighbor.is_group = res->is_group;
-	strlcpy(neighbor.shutcomm, res->shutcomm, sizeof(neighbor.shutcomm));
+	strlcpy(neighbor.reason, res->reason, sizeof(neighbor.reason));
 
 	switch (res->action) {
 	case SHOW_MRT:
@@ -213,6 +213,12 @@ main(int argc, char *argv[])
 	case SHOW_INTERFACE:
 		imsg_compose(ibuf, IMSG_CTL_SHOW_INTERFACE, 0, 0, -1, NULL, 0);
 		break;
+	case SHOW_SET:
+		imsg_compose(ibuf, IMSG_CTL_SHOW_SET, 0, 0, -1, NULL, 0);
+		break;
+	case SHOW_RTR:
+		imsg_compose(ibuf, IMSG_CTL_SHOW_RTR, 0, 0, -1, NULL, 0);
+		break;
 	case SHOW_NEIGHBOR:
 	case SHOW_NEIGHBOR_TIMERS:
 	case SHOW_NEIGHBOR_TERSE:
@@ -246,8 +252,12 @@ main(int argc, char *argv[])
 		imsg_compose(ibuf, IMSG_CTL_SHOW_RIB_MEM, 0, 0, -1, NULL, 0);
 		break;
 	case RELOAD:
-		imsg_compose(ibuf, IMSG_CTL_RELOAD, 0, 0, -1, NULL, 0);
-		printf("reload request sent.\n");
+		imsg_compose(ibuf, IMSG_CTL_RELOAD, 0, 0, -1,
+		    res->reason, sizeof(res->reason));
+		if (res->reason[0])
+			printf("reload request sent: %s\n", res->reason);
+		else
+			printf("reload request sent.\n");
 		break;
 	case FIB:
 		errx(1, "action==FIB");
@@ -386,17 +396,19 @@ int
 show(struct imsg *imsg, struct parse_result *res)
 {
 	struct peer		*p;
-	struct ctl_timer	*t;
+	struct ctl_timer	 t;
 	struct ctl_show_interface	*iface;
 	struct ctl_show_nexthop	*nh;
+	struct ctl_show_set	 set;
+	struct ctl_show_rtr	 rtr;
 	struct kroute_full	*kf;
 	struct ktable		*kt;
 	struct ctl_show_rib	 rib;
+	struct rde_memstats	 stats;
+	struct rde_hashstats	 hash;
 	u_char			*asdata;
-	struct rde_memstats	stats;
-	struct rde_hashstats	hash;
-	u_int			rescode, ilen;
-	size_t			aslen;
+	u_int			 rescode, ilen;
+	size_t			 aslen;
 
 	switch (imsg->hdr.type) {
 	case IMSG_CTL_SHOW_NEIGHBOR:
@@ -404,9 +416,11 @@ show(struct imsg *imsg, struct parse_result *res)
 		output->neighbor(p, res);
 		break;
 	case IMSG_CTL_SHOW_TIMER:
-		t = imsg->data;
-		if (t->type > 0 && t->type < Timer_Max)
-			output->timer(t);
+		if (imsg->hdr.len < IMSG_HEADER_SIZE + sizeof(t))
+			errx(1, "wrong imsg len");
+		memcpy(&t, imsg->data, sizeof(t));
+		if (t.type > 0 && t.type < Timer_Max)
+			output->timer(&t);
 		break;
 	case IMSG_CTL_SHOW_INTERFACE:
 		iface = imsg->data;
@@ -452,16 +466,32 @@ show(struct imsg *imsg, struct parse_result *res)
 			warnx("bad IMSG_CTL_SHOW_RIB_ATTR received");
 			break;
 		}
-		output->attr(imsg->data, ilen, res);
+		output->attr(imsg->data, ilen, res->flags);
 		break;
 	case IMSG_CTL_SHOW_RIB_MEM:
+		if (imsg->hdr.len < IMSG_HEADER_SIZE + sizeof(stats))
+			errx(1, "wrong imsg len");
 		memcpy(&stats, imsg->data, sizeof(stats));
 		output->rib_mem(&stats);
 		break;
 	case IMSG_CTL_SHOW_RIB_HASH:
+		if (imsg->hdr.len < IMSG_HEADER_SIZE + sizeof(hash))
+			errx(1, "wrong imsg len");
 		memcpy(&hash, imsg->data, sizeof(hash));
 		output->rib_hash(&hash);
 		break;
+	case IMSG_CTL_SHOW_SET:
+		if (imsg->hdr.len < IMSG_HEADER_SIZE + sizeof(set))
+			errx(1, "wrong imsg len");
+		memcpy(&set, imsg->data, sizeof(set));
+		output->set(&set);
+		break;
+	case IMSG_CTL_SHOW_RTR:
+		if (imsg->hdr.len < IMSG_HEADER_SIZE + sizeof(rtr))
+			errx(1, "wrong imsg len");
+		memcpy(&rtr, imsg->data, sizeof(rtr));
+		output->rtr(&rtr);
+		break;
 	case IMSG_CTL_RESULT:
 		if (imsg->hdr.len != IMSG_HEADER_SIZE + sizeof(rescode)) {
 			warnx("got IMSG_CTL_RESULT with wrong len");
@@ -480,6 +510,20 @@ show(struct imsg *imsg, struct parse_result *res)
 	return (0);
 }
 
+time_t
+get_monotime(time_t t)
+{
+	struct timespec ts;
+
+	if (t == 0)
+		return -1;
+	if (clock_gettime(CLOCK_MONOTONIC, &ts) != 0)
+		err(1, "clock_gettime");
+	if (t > ts.tv_sec)	/* time in the future is not possible */
+		t = ts.tv_sec;
+	return (ts.tv_sec - t);
+}
+
 char *
 fmt_peer(const char *descr, const struct bgpd_addr *remote_addr,
     int masklen)
@@ -566,16 +610,12 @@ fmt_timeframe(time_t t)
 const char *
 fmt_monotime(time_t t)
 {
-	struct timespec ts;
+	t = get_monotime(t);
 
-	if (t == 0)
+	if (t == -1)
 		return ("Never");
 
-	if (clock_gettime(CLOCK_MONOTONIC, &ts) != 0)
-		err(1, "clock_gettime");
-	if (t > ts.tv_sec)	/* time in the future is not possible */
-		t = ts.tv_sec;
-	return (fmt_timeframe(ts.tv_sec - t));
+	return (fmt_timeframe(t));
 }
 
 const char *
@@ -973,6 +1013,23 @@ fmt_ext_community(u_int8_t *data)
 	}
 }
 
+const char *
+fmt_set_type(struct ctl_show_set *set)
+{
+	switch (set->type) {
+	case ROA_SET:
+		return "ROA";
+	case PREFIX_SET:
+		return "PREFIX";
+	case ORIGIN_SET:
+		return "ORIGIN";
+	case ASNUM_SET:
+		return "ASNUM";
+	default:
+		return "BULA";
+	}
+}
+
 void
 send_filterset(struct imsgbuf *i, struct filter_set_head *set)
 {
@@ -1134,7 +1191,7 @@ show_mrt_dump(struct mrt_rib *mr, struct mrt_peer *mp, void *arg)
 		if (req->flags & F_CTL_DETAIL) {
 			for (j = 0; j < mre->nattrs; j++)
 				output->attr(mre->attrs[j].attr,
-				    mre->attrs[j].attr_len, &res);
+				    mre->attrs[j].attr_len, req->flags);
 		}
 	}
 }
@@ -1294,6 +1351,13 @@ print_capability(u_int8_t capa_code, u_char *p, u_int8_t len)
 		} else
 			printf("bad length");
 		break;
+	case CAPA_ADD_PATH:
+		printf("add-path capability");
+		/* XXX there is more needed here */
+		break;
+	case CAPA_ENHANCED_RR:
+		printf("enhanced route refresh capability");
+		break;
 	default:
 		printf("unknown capability %u length %u", capa_code, len);
 		break;
@@ -1459,8 +1523,8 @@ show_mrt_notification(u_char *p, u_int16_t len)
 {
 	u_int16_t i;
 	u_int8_t errcode, subcode;
-	size_t shutcomm_len;
-	char shutcomm[SHUT_COMM_LEN];
+	size_t reason_len;
+	char reason[REASON_LEN];
 
 	memcpy(&errcode, p, sizeof(errcode));
 	p += sizeof(errcode);
@@ -1476,22 +1540,22 @@ show_mrt_notification(u_char *p, u_int16_t len)
 	if (errcode == ERR_CEASE && (subcode == ERR_CEASE_ADMIN_DOWN ||
 	    subcode == ERR_CEASE_ADMIN_RESET)) {
 		if (len > 1) {
-			shutcomm_len = *p++;
+			reason_len = *p++;
 			len--;
-			if (len < shutcomm_len) {
+			if (len < reason_len) {
 				printf("truncated shutdown reason");
 				return;
 			}
-			if (shutcomm_len > SHUT_COMM_LEN - 1) {
+			if (reason_len > REASON_LEN - 1) {
 				printf("overly long shutdown reason");
 				return;
 			}
-			memcpy(shutcomm, p, shutcomm_len);
-			shutcomm[shutcomm_len] = '\0';
+			memcpy(reason, p, reason_len);
+			reason[reason_len] = '\0';
 			printf("shutdown reason: \"%s\"",
-			    log_shutcomm(shutcomm));
-			p += shutcomm_len;
-			len -= shutcomm_len;
+			    log_reason(reason));
+			p += reason_len;
+			len -= reason_len;
 		}
 	}
 	if (errcode == ERR_OPEN && subcode == ERR_OPEN_CAPA) {
@@ -1518,7 +1582,7 @@ show_mrt_notification(u_char *p, u_int16_t len)
 
 /* XXX this function does not handle JSON output */
 static void
-show_mrt_update(u_char *p, u_int16_t len)
+show_mrt_update(u_char *p, u_int16_t len, int reqflags)
 {
 	struct bgpd_addr prefix;
 	int pos;
@@ -1587,7 +1651,7 @@ show_mrt_update(u_char *p, u_int16_t len)
 			attrlen += 1 + 2;
 		}
 
-		output->attr(p, attrlen, 0);
+		output->attr(p, attrlen, reqflags);
 		p += attrlen;
 		alen -= attrlen;
 		len -= attrlen;
@@ -1617,6 +1681,7 @@ show_mrt_msg(struct mrt_bgp_msg *mm, void *arg)
 	u_char *p;
 	u_int16_t len;
 	u_int8_t type;
+	struct ctl_show_rib_request *req = arg;
 
 	printf("%s %s[%u] -> ", fmt_time(&mm->time),
 	    log_addr(&mm->src), mm->src_as);
@@ -1670,7 +1735,7 @@ show_mrt_msg(struct mrt_bgp_msg *mm, void *arg)
 			printf("illegal length: %u byte\n", len);
 			return;
 		}
-		show_mrt_update(p, len - MSGSIZE_HEADER);
+		show_mrt_update(p, len - MSGSIZE_HEADER, req->flags);
 		break;
 	case KEEPALIVE:
 		printf("%s ", msgtypenames[type]);
