@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.c,v 1.104 2020/05/12 13:26:02 claudio Exp $ */
+/*	$OpenBSD: parser.c,v 1.106 2021/02/16 08:30:21 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -136,6 +136,8 @@ static const struct token t_show[] = {
 	{ KEYWORD,	"tables",	SHOW_FIB_TABLES, NULL},
 	{ KEYWORD,	"ip",		NONE,		t_show_ip},
 	{ KEYWORD,	"summary",	SHOW_SUMMARY,	t_show_summary},
+	{ KEYWORD,	"sets",		SHOW_SET,	NULL},
+	{ KEYWORD,	"rtr",		SHOW_RTR,	NULL},
 	{ KEYWORD,	"mrt",		SHOW_MRT,	t_show_mrt},
 	{ ENDTOKEN,	"",		NONE,		NULL}
 };
