@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.54 2020/05/10 13:38:46 deraadt Exp $ */
+/*	$OpenBSD: util.c,v 1.61 2021/02/16 08:29:16 claudio Exp $ */
 
 /*
  * Copyright (c) 2006 Claudio Jeker <claudio@openbsd.org>
@@ -38,28 +38,18 @@ const char *
 log_addr(const struct bgpd_addr *addr)
 {
 	static char	buf[74];
-	char		tbuf[40];
+	struct sockaddr *sa;
+	socklen_t	len;
 
+	sa = addr2sa(addr, 0, &len);
 	switch (addr->aid) {
 	case AID_INET:
 	case AID_INET6:
-		if (inet_ntop(aid2af(addr->aid), &addr->ba, buf,
-		    sizeof(buf)) == NULL)
-			return ("?");
-		return (buf);
+		return log_sockaddr(sa, len);
 	case AID_VPN_IPv4:
-		if (inet_ntop(AF_INET, &addr->vpn4.addr, tbuf,
-		    sizeof(tbuf)) == NULL)
-			return ("?");
-		snprintf(buf, sizeof(buf), "%s %s", log_rd(addr->vpn4.rd),
-		    tbuf);
-		return (buf);
 	case AID_VPN_IPv6:
-		if (inet_ntop(aid2af(addr->aid), &addr->vpn6.addr, tbuf,
-		    sizeof(tbuf)) == NULL)
-			return ("?");
-		snprintf(buf, sizeof(buf), "%s %s", log_rd(addr->vpn6.rd),
-		    tbuf);
+		snprintf(buf, sizeof(buf), "%s %s", log_rd(addr->rd),
+		    log_sockaddr(sa, len));
 		return (buf);
 	}
 	return ("???");
@@ -69,7 +59,6 @@ const char *
 log_in6addr(const struct in6_addr *addr)
 {
 	struct sockaddr_in6	sa_in6;
-	u_int16_t		tmp16;
 
 	bzero(&sa_in6, sizeof(sa_in6));
 	sa_in6.sin6_family = AF_INET6;
@@ -79,6 +68,7 @@ log_in6addr(const struct in6_addr *addr)
 	/* XXX thanks, KAME, for this ugliness... adopted from route/show.c */
 	if (IN6_IS_ADDR_LINKLOCAL(&sa_in6.sin6_addr) ||
 	    IN6_IS_ADDR_MC_LINKLOCAL(&sa_in6.sin6_addr)) {
+		u_int16_t tmp16;
 		memcpy(&tmp16, &sa_in6.sin6_addr.s6_addr[2], sizeof(tmp16));
 		sa_in6.sin6_scope_id = ntohs(tmp16);
 		sa_in6.sin6_addr.s6_addr[2] = 0;
@@ -94,7 +84,7 @@ log_sockaddr(struct sockaddr *sa, socklen_t len)
 {
 	static char	buf[NI_MAXHOST];
 
-	if (getnameinfo(sa, len, buf, sizeof(buf), NULL, 0,
+	if (sa == NULL || getnameinfo(sa, len, buf, sizeof(buf), NULL, 0,
 	    NI_NUMERICHOST))
 		return ("(unknown)");
 	else
@@ -171,6 +161,38 @@ log_reason(const char *communication) {
 	return buf;
 }
 
+const char *
+log_rtr_error(enum rtr_error err)
+{
+	static char buf[20];
+
+	switch (err) {
+	case NO_ERROR:
+		return "No Error";
+	case CORRUPT_DATA:
+		return "Corrupt Data";
+	case INTERNAL_ERROR:
+		return "Internal Error";
+	case NO_DATA_AVAILABLE:
+		return "No Data Available";
+	case INVALID_REQUEST:
+		return "Invalid Request";
+	case UNSUPP_PROTOCOL_VERS:
+		return "Unsupported Protocol Version";
+	case UNSUPP_PDU_TYPE:
+		return "Unsupported PDU Type";
+	case UNK_REC_WDRAWL:
+		return "Withdrawl of Unknown Record";
+	case DUP_REC_RECV:
+		return "Duplicate Announcement Received";
+	case UNEXP_PROTOCOL_VERS:
+		return "Unexpected Protocol Version";
+	default:
+		snprintf(buf, sizeof(buf), "unknown %u", err);
+		return buf;
+	}
+}
+
 const char *
 aspath_delim(u_int8_t seg_type, int closing)
 {
@@ -338,7 +360,7 @@ aspath_extract(const void *seg, int pos)
  * Verify that the aspath is correctly encoded.
  */
 int
-aspath_verify(void *data, u_int16_t len, int as4byte)
+aspath_verify(void *data, u_int16_t len, int as4byte, int noset)
 {
 	u_int8_t	*seg = data;
 	u_int16_t	 seg_size, as_size = 2;
@@ -372,6 +394,12 @@ aspath_verify(void *data, u_int16_t len, int as4byte)
 		 */
 		if (seg_type == AS_CONFED_SEQUENCE || seg_type == AS_CONFED_SET)
 			error = AS_ERR_SOFT;
+		/*
+		 * If AS_SET filtering (RFC6472) is on, error out on AS_SET
+		 * as well.
+		 */
+		if (noset && seg_type == AS_SET)
+			error = AS_ERR_SOFT;
 		if (seg_type != AS_SET && seg_type != AS_SEQUENCE &&
 		    seg_type != AS_CONFED_SEQUENCE && seg_type != AS_CONFED_SET)
 			return (AS_ERR_TYPE);
@@ -539,8 +567,8 @@ nlri_get_vpn4(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 	do {
 		if (len - plen < 3 || pfxlen < 3 * 8)
 			return (-1);
-		if (prefix->vpn4.labellen + 3U >
-		    sizeof(prefix->vpn4.labelstack))
+		if (prefix->labellen + 3U >
+		    sizeof(prefix->labelstack))
 			return (-1);
 		if (withdraw) {
 			/* on withdraw ignore the labelstack all together */
@@ -548,13 +576,13 @@ nlri_get_vpn4(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 			pfxlen -= 3 * 8;
 			break;
 		}
-		prefix->vpn4.labelstack[prefix->vpn4.labellen++] = *p++;
-		prefix->vpn4.labelstack[prefix->vpn4.labellen++] = *p++;
-		prefix->vpn4.labelstack[prefix->vpn4.labellen] = *p++;
-		if (prefix->vpn4.labelstack[prefix->vpn4.labellen] &
+		prefix->labelstack[prefix->labellen++] = *p++;
+		prefix->labelstack[prefix->labellen++] = *p++;
+		prefix->labelstack[prefix->labellen] = *p++;
+		if (prefix->labelstack[prefix->labellen] &
 		    BGP_MPLS_BOS)
 			done = 1;
-		prefix->vpn4.labellen++;
+		prefix->labellen++;
 		plen += 3;
 		pfxlen -= 3 * 8;
 	} while (!done);
@@ -563,7 +591,7 @@ nlri_get_vpn4(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 	if (len - plen < (int)sizeof(u_int64_t) ||
 	    pfxlen < sizeof(u_int64_t) * 8)
 		return (-1);
-	memcpy(&prefix->vpn4.rd, p, sizeof(u_int64_t));
+	memcpy(&prefix->rd, p, sizeof(u_int64_t));
 	pfxlen -= sizeof(u_int64_t) * 8;
 	p += sizeof(u_int64_t);
 	plen += sizeof(u_int64_t);
@@ -574,8 +602,8 @@ nlri_get_vpn4(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 
 	if (pfxlen > 32)
 		return (-1);
-	if ((rv = extract_prefix(p, len, &prefix->vpn4.addr,
-	    pfxlen, sizeof(prefix->vpn4.addr))) == -1)
+	if ((rv = extract_prefix(p, len, &prefix->v4,
+	    pfxlen, sizeof(prefix->v4))) == -1)
 		return (-1);
 
 	return (plen + rv);
@@ -602,8 +630,8 @@ nlri_get_vpn6(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 	do {
 		if (len - plen < 3 || pfxlen < 3 * 8)
 			return (-1);
-		if (prefix->vpn6.labellen + 3U >
-		    sizeof(prefix->vpn6.labelstack))
+		if (prefix->labellen + 3U >
+		    sizeof(prefix->labelstack))
 			return (-1);
 		if (withdraw) {
 			/* on withdraw ignore the labelstack all together */
@@ -612,13 +640,13 @@ nlri_get_vpn6(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 			break;
 		}
 
-		prefix->vpn6.labelstack[prefix->vpn6.labellen++] = *p++;
-		prefix->vpn6.labelstack[prefix->vpn6.labellen++] = *p++;
-		prefix->vpn6.labelstack[prefix->vpn6.labellen] = *p++;
-		if (prefix->vpn6.labelstack[prefix->vpn6.labellen] &
+		prefix->labelstack[prefix->labellen++] = *p++;
+		prefix->labelstack[prefix->labellen++] = *p++;
+		prefix->labelstack[prefix->labellen] = *p++;
+		if (prefix->labelstack[prefix->labellen] &
 		    BGP_MPLS_BOS)
 			done = 1;
-		prefix->vpn6.labellen++;
+		prefix->labellen++;
 		plen += 3;
 		pfxlen -= 3 * 8;
 	} while (!done);
@@ -628,7 +656,7 @@ nlri_get_vpn6(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 	    pfxlen < sizeof(u_int64_t) * 8)
 		return (-1);
 
-	memcpy(&prefix->vpn6.rd, p, sizeof(u_int64_t));
+	memcpy(&prefix->rd, p, sizeof(u_int64_t));
 	pfxlen -= sizeof(u_int64_t) * 8;
 	p += sizeof(u_int64_t);
 	plen += sizeof(u_int64_t);
@@ -640,8 +668,8 @@ nlri_get_vpn6(u_char *p, u_int16_t len, struct bgpd_addr *prefix,
 	if (pfxlen > 128)
 		return (-1);
 
-	if ((rv = extract_prefix(p, len, &prefix->vpn6.addr,
-	    pfxlen, sizeof(prefix->vpn6.addr))) == -1)
+	if ((rv = extract_prefix(p, len, &prefix->v6,
+	    pfxlen, sizeof(prefix->v6))) == -1)
 		return (-1);
 
 	return (plen + rv);
@@ -665,6 +693,12 @@ prefix_compare(const struct bgpd_addr *a, const struct bgpd_addr *b,
 		return (a->aid - b->aid);
 
 	switch (a->aid) {
+	case AID_VPN_IPv4:
+		if (be64toh(a->rd) > be64toh(b->rd))
+			return (1);
+		if (be64toh(a->rd) < be64toh(b->rd))
+			return (-1);
+		/* FALLTHROUGH */
 	case AID_INET:
 		if (prefixlen == 0)
 			return (0);
@@ -673,9 +707,17 @@ prefix_compare(const struct bgpd_addr *a, const struct bgpd_addr *b,
 		mask = htonl(prefixlen2mask(prefixlen));
 		aa = ntohl(a->v4.s_addr & mask);
 		ba = ntohl(b->v4.s_addr & mask);
-		if (aa != ba)
-			return (aa - ba);
-		return (0);
+		if (aa > ba)
+			return (1);
+		if (aa < ba)
+			return (-1);
+		break;
+	case AID_VPN_IPv6:
+		if (be64toh(a->rd) > be64toh(b->rd))
+			return (1);
+		if (be64toh(a->rd) < be64toh(b->rd))
+			return (-1);
+		/* FALLTHROUGH */
 	case AID_INET6:
 		if (prefixlen == 0)
 			return (0);
@@ -692,53 +734,20 @@ prefix_compare(const struct bgpd_addr *a, const struct bgpd_addr *b,
 				return ((a->v6.s6_addr[prefixlen / 8] & m) -
 				    (b->v6.s6_addr[prefixlen / 8] & m));
 		}
-		return (0);
-	case AID_VPN_IPv4:
-		if (prefixlen > 32)
-			return (-1);
-		if (be64toh(a->vpn4.rd) > be64toh(b->vpn4.rd))
-			return (1);
-		if (be64toh(a->vpn4.rd) < be64toh(b->vpn4.rd))
-			return (-1);
-		mask = htonl(prefixlen2mask(prefixlen));
-		aa = ntohl(a->vpn4.addr.s_addr & mask);
-		ba = ntohl(b->vpn4.addr.s_addr & mask);
-		if (aa != ba)
-			return (aa - ba);
-		if (a->vpn4.labellen > b->vpn4.labellen)
-			return (1);
-		if (a->vpn4.labellen < b->vpn4.labellen)
-			return (-1);
-		return (memcmp(a->vpn4.labelstack, b->vpn4.labelstack,
-		    a->vpn4.labellen));
-	case AID_VPN_IPv6:
-		if (prefixlen > 128)
-			return (-1);
-		if (be64toh(a->vpn6.rd) > be64toh(b->vpn6.rd))
-			return (1);
-		if (be64toh(a->vpn6.rd) < be64toh(b->vpn6.rd))
-			return (-1);
-		for (i = 0; i < prefixlen / 8; i++)
-			if (a->vpn6.addr.s6_addr[i] != b->vpn6.addr.s6_addr[i])
-				return (a->vpn6.addr.s6_addr[i] -
-				    b->vpn6.addr.s6_addr[i]);
-		i = prefixlen % 8;
-		if (i) {
-			m = 0xff00 >> i;
-			if ((a->vpn6.addr.s6_addr[prefixlen / 8] & m) !=
-			    (b->vpn6.addr.s6_addr[prefixlen / 8] & m))
-				return ((a->vpn6.addr.s6_addr[prefixlen / 8] &
-				    m) - (b->vpn6.addr.s6_addr[prefixlen / 8] &
-				    m));
-		}
-		if (a->vpn6.labellen > b->vpn6.labellen)
-			return (1);
-		if (a->vpn6.labellen < b->vpn6.labellen)
-			return (-1);
-		return (memcmp(a->vpn6.labelstack, b->vpn6.labelstack,
-		    a->vpn6.labellen));
+		break;
+	default:
+		return (-1);
 	}
-	return (-1);
+
+	if (a->aid == AID_VPN_IPv4 || a->aid == AID_VPN_IPv6) {
+		if (a->labellen > b->labellen)
+			return (1);
+		if (a->labellen < b->labellen)
+			return (-1);
+		return (memcmp(a->labelstack, b->labelstack, a->labellen));
+	}
+	return (0);
+
 }
 
 in_addr_t
@@ -837,8 +846,12 @@ af2aid(sa_family_t af, u_int8_t safi, u_int8_t *aid)
 	return (-1);
 }
 
+/*
+ * Convert a struct bgpd_addr into a struct sockaddr. For VPN addresses
+ * the included label stack is ignored and needs to be handled by the caller.
+ */
 struct sockaddr *
-addr2sa(struct bgpd_addr *addr, u_int16_t port, socklen_t *len)
+addr2sa(const struct bgpd_addr *addr, u_int16_t port, socklen_t *len)
 {
 	static struct sockaddr_storage	 ss;
 	struct sockaddr_in		*sa_in = (struct sockaddr_in *)&ss;
@@ -850,12 +863,14 @@ addr2sa(struct bgpd_addr *addr, u_int16_t port, socklen_t *len)
 	bzero(&ss, sizeof(ss));
 	switch (addr->aid) {
 	case AID_INET:
+	case AID_VPN_IPv4:
 		sa_in->sin_family = AF_INET;
 		sa_in->sin_addr.s_addr = addr->v4.s_addr;
 		sa_in->sin_port = htons(port);
 		*len = sizeof(struct sockaddr_in);
 		break;
 	case AID_INET6:
+	case AID_VPN_IPv6:
 		sa_in6->sin6_family = AF_INET6;
 		memcpy(&sa_in6->sin6_addr, &addr->v6,
 		    sizeof(sa_in6->sin6_addr));
